#!/usr/bin/perl

# /usr/local/bin/matrixuhr.pl
# Dieses Programm sendet Daten über die RS-232-Schnittstelle zu einer angeschlossenen Matrix-Uhr:
# Alarme, Sensorwerte, Geburtstage, spezielle Alarme, die Uhrzeit, eine Textnachricht, Wetter- und Kraftstoffdaten.
# Zusätzlich werden regelmäßig einige Status-Informationen an den MQTT-Server gesendet.
# Scott-Falk Hühn, 15.02.2024

use strict;
use warnings;
use Time::HiRes qw(time);     # wird für Zeitoperation im Millisekundenbereich benötigt
use Net::MQTT::Simple;        # wird für die Kommunikation mit dem MQTT-Server benötigt
use Encode;                   # wird zur UTF-8-Konvertierung benötigt
use JSON::XS;                 # wird zum Lesen der Wetter- und Kraftstoffdaten benötigt

my $senif = "/dev/serial0";   # Serielle Schnittstelle des Raspberry Pi
my $sendir = "/mnt/ram";      # Verzeichnis für flüchtige Sensor- und Alarmdaten
my $cfgdir = "/srv/sensor";   # Verzeichnis für Konfigurationsdaten
my $cfgname = "mqttmu";       # Name der MQTT-Konfigurationsdatei
my $senlname = "senlmu";      # Name der Sensorlistendatei
my $senname = "senmu";        # Name der Sensordatei
my $almname = "almu";         # Name einer Alarmdatei
my $bdname = "bdaymu";        # Name der Geburtstagsdatei
my $timename = "timemu";      # Name der Zeitsynchronisierungsdatei
my $callname = "callmu";      # Name der Anruf-Konfigurationsdatei
my $wename = "weatmu";        # Name der Wetter-Konfigurationsdatei
my $wcname = "wc??mu";        # Name der Wetterlage-Datei
my $icname = "iconmu";        # Name der Icondatei
my $jsname = "weather.json";  # Name der Wetterdatei
my $w1name = "s32";           # Name der 1-Wire-Sensorwert-Datei
my $sdname = "sdatmu";        # Name der MQTT-Konfigurationsdatei für spezielle Daten
my $mqtt_lmsg = "offline";    # MQTT LWT-Message setzen
my $mqtt_smsg = "online";     # MQTT LWT-Start-Message setzen
my $test;                     # Flag für Test-Modus, wird durch den Parameter "t" oder "T" beim Start aktiviert
my $mqtt_use;                 # Verwendung des MQTT-Servers, wird aus Konfigurationsdatei gelesen (0 = aus, 1 = ein)
my $mqtt_addr;                # IP-Adresse des MQTT-Servers, wird aus Konfigurationsdatei gelesen
my $mqtt_port;                # Port des MQTT-Servers, wird aus Konfigurationsdatei gelesen
my $mqtt_vali;                # Gültigkeitsdauer der Daten, wird aus Konfigurationsdatei gelesen
my $mqtt_user;                # Username für MQTT-Server
my $mqtt_pass;                # Passwort für MQTT-Server
my $mqtt_lwt;                 # LWT Topic
my $mqtt_upt;                 # Uptime Topic
my $mqtt_rssi;                # RSSI Topic
my $mqtt_load;                # Load Topic
my $mqtt;                     # Verbindung zum MQTT-Server
my $time_use;                 # Verwendung der Zeitsynchronisierung, wird aus Konfigurationsdatei gelesen (1 = ein)
my $time_int;                 # Synchronisierungsintervall, wird aus Konfigurationsdatei gelesen (1-60)
my $we_use;                   # Verwendung der Wetterdatenabfrage, wird aus Konfigurationsdatei gelesen (1 = ein)
my $we_lng;                   # Sprache der Wetterlagetexte
my $we_res;                   # empfangenes Datenpaket mit den Wetterdaten im JSON-Format
my $we_time;                  # Zeitstempel des letzten empfangenen Datenpaketes
my @we_vals;                  # Array mit den ermittelten Wetterdaten
my @sentlist;                 # Array mit den MQTT Topics der verwendeten Sensoren (maximal 12 Einträge)
my @senflist;                 # Array mit den Formaten der verwendeten Sensoren (maximal 12 Einträge)
my @senvals;                  # Array mit den empfangenen Sensorwerten (maximal 12 Einträge)
my @sentimes;                 # Array mit den Zeitstempeln der empfangenen Sensorwerte (maximal 12 Einträge)
my @sensors;                  # Array mit den auszugebenden Sensordatenfeldern
my @almparam;                 # Array mit den Alarm-Parametern (8 Einträge)
my @almtlist;                 # Array mit den MQTT Topics bzw. Bedingungen der Alarme (8 Einträge)
my @almtflag;                 # Array mit den Alarm-Timer-Flags (8 Einträge; nur 0, 4, 5, 6, 7 wird genutzt)
my @bdays;                    # Array mit den Geburtstagen
my @wecndn;                   # Array mit den Wetterlagenummern
my @wecndt;                   # Array mit den Wetterlagetexten
my @icons;                    # Array mit der Iconliste
my @we_str;                   # Array mit den einzelnen Wörter des Wetterlage-Textes
my @fu_vals;                  # Array mit den ermittelten Kraftstoffdaten
my $SER;                      # Dateihandle der seriellen Schnittstelle
my $CFG;                      # Dateihandle einer Konfigurationsdatei
my $MES;                      # Dateihandle der Textnachrichtendatei
my $SEN;                      # Dateihandle einer Sensordatei (1-Wire)
my $WEA;                      # Dateihandle der Wetterdatei
my $count;                    # Zyklus-Zähler:
                              # 1 - $senmax = Sensorwerte ausgeben
                              # $senmax + 1 = Geburtstage ausgeben und 1-Wire-Sensor lesen
                              # $senmax + 2 = spezielle Alarme ausgeben
                              # $senmax + 3 = Uhrzeit und Textnachricht ausgeben
                              # $senmax + 4 = Wetterdaten einlesen
my $sencnt;                   # Sensornummernzähler (wird bei der Initialisierung als Datensatzzähler verwendet)
my $sencn2;                   # weiterer Datensatzzähler für die Initialisierung
my $sendat;                   # auszugebender Sensordatensatz
my $senmax;                   # Anzahl der auszugebenden Sensorwerte
my $seline;                   # Inhalt einer Zeile der Sensordatenliste
my $semode;                   # Sensormodus (0 = Zeichen normal ausgeben, 1 = Zeichen als Sensorname lesen)
my $sename;                   # auszugebender Sensorname
my $senval;                   # auszugebender Sensorwert
my $setime;                   # Zeitstempel des auszugebenden Sensorwertes
my $senok;                    # Flag für die Ermittlung eines Sensorwertes
my $bdline;                   # Inhalt einer Zeile der Geburtstagsliste
my $bdnumb;                   # Geburtstagsnummer, wird wie eine Sensornummer behandelt
my $bdayflag;                 # Geburtstags-Flag, wird gesetzt, wenn Geburtstage anstehen
my $lasttime;                 # Zeitstempel des letzten gesendeten Datenpaketes
my $lastsync;                 # Zeitstempel der letzten Synchronisierung
my $laststat;                 # Zeitstempel der letzten MQTT-Statusmeldung
my $uptime;                   # aktuelle Systemlaufzeit
my $rssival;                  # aktuelle WLAN-Empfangsstärke
my $loadval;                  # aktuelle Systemlast
my $curtim;                   # aktuelle Zeit
my $sentim;                   # gesendete Zeitinformation
my $srtime;                   # Sonnenaufgangszeit
my $sstime;                   # Sonnenuntergangszeit
my $alline;                   # Bedingung für spezielle Alarme
my $comnam;                   # Vergleichssensor für spezielle Alarme
my $comval;                   # Vergleichswert für spezielle Alarme
my $comtim;                   # Vergleichswert-Zeitstempel für spezielle Alarme
my $meline;                   # Inhalt der Textnachrichtendatei (muss einzeilig sein)
my $topic;                    # Zwischenspeicher für MQTT-Topic
my $format;                   # Zwischenspeicher für Datenformat
my $temp;                     # temporärer Zwischenspeicher
my $temp2;                    # temporärer Zwischenspeicher
my $temp3;                    # temporärer Zwischenspeicher
my $temp4;                    # temporärer Zwischenspeicher
my $tstr;                     # temporärer Zwischenspeicher für Strings
my $call_use;                 # Verwendung der Anrufanzeige (0 = aus, 1-8 = verwendete Alarmnummer)
my $call_numb;                # Anrufparameter Nummer (eigene Rufnummer oder leer für alle Nummern)
my $call_evnt;                # Anrufparameter Ereignis (wird über MQTT übermittelt)
my $call_name;                # Anrufparameter Name (wird über MQTT übermittelt)
my $call_extn;                # Anrufparameter Externe Nummer (wird über MQTT übermittelt)
my $call_intn;                # Anrufparameter Interne Nummer (wird über MQTT übermittelt)
my $w1_val;                   # 1-Wire-Temperaturwert
my $sec;                      # Sekunde (Zwischenspeicher für Zeitabfrage)
my $min;                      # Minute (Zwischenspeicher für Zeitabfrage)
my $hou;                      # Stunde (Zwischenspeicher für Zeitabfrage)
my $day;                      # Tag (Zwischenspeicher für Zeitabfrage)
my $mon;                      # Monat (Zwischenspeicher für Zeitabfrage)
my $yea;                      # Jahr (Zwischenspeicher für Zeitabfrage)
my $wday;                     # Wochentag (Zwischenspeicher für Zeitabfrage)
my $yday;                     # Jahrestag (Zwischenspeicher für Zeitabfrage)
my $dst;                      # Sommerzeit (Zwischenspeicher für Zeitabfrage)
my $sdtop1;                   # MQTT Topic für Wetterdaten
my $sdtop2;                   # MQTT Topic für Kraftstoffdaten
my $sdtop3;                   # MQTT Topic für Nachrichten
my $mqttwval;                 # über MQTT empfangene Wetterdaten
my $mqttfval;                 # über MQTT empfangene Kraftstoffdaten
my $mqttftim;                 # Zeitstempel der MQTT-Kraftstoffdaten
my $mqttmval;                 # über MQTT empfangene Textnachricht
my $jsonbuf;                  # Puffer für dekodierte JSON-Daten
my $i;                        # Zählervariable
my $j;                        # Zählervariable

$test = 0;                                                    # Test zunächst ausschalten
if (defined $ARGV[0]) {                                       # wenn Parameter 1 angegeben
  if ($ARGV[0] eq 't' or $ARGV[0] eq 'T') { $test = 1; }      # wenn Zeichen 't' oder 'T' -> Test einschalten
}

if ($test) { print "Konfiguration lesen...\n"; }

if (open($CFG, "<$cfgdir/$cfgname")) {                        # wenn MQTT-Konfigurationsdatei vorhanden -> öffnen
  $mqtt_use = <$CFG>;                                         # MQTT-Aktivierung lesen
  $mqtt_addr = <$CFG>;                                        # MQTT-Server-Adresse lesen
  $mqtt_port = <$CFG>;                                        # MQTT-Server-Port lesen
  $mqtt_vali = <$CFG>;                                        # MQTT-Gültigkeitsdauer lesen
  $mqtt_user = <$CFG>;                                        # Username für MQTT-Server lesen
  $mqtt_pass = <$CFG>;                                        # Passwort für MQTT-Server lesen
  $mqtt_lwt = <$CFG>;                                         # LWT Topic lesen
  $mqtt_upt = <$CFG>;                                         # Uptime Topic lesen
  $mqtt_rssi = <$CFG>;                                        # RSSI Topic lesen
  $mqtt_load = <$CFG>;                                        # Load Topic lesen
  close($CFG);                                                # MQTT-Konfigurationsdatei schließen
}
if ((defined $mqtt_use) && (defined $mqtt_addr) && (defined $mqtt_port)) { # wenn Hauptparameter vorhanden
  chomp($mqtt_use);                                           # Zeilenendezeichen der Aktivierung entfernen
  chomp($mqtt_addr);                                          # Zeilenendezeichen der Adresse entfernen
  chomp($mqtt_port);                                          # Zeilenendezeichen des Ports entfernen
  chomp($mqtt_vali);                                          # Zeilenendezeichen der Gültigkeitsdauer entfernen
  if (!defined $mqtt_user) { $mqtt_user = ''; }               # Username setzen (bei Update)
  if (!defined $mqtt_pass) { $mqtt_pass = ''; }               # Passwort setzen (bei Update)
  if (!defined $mqtt_lwt) { $mqtt_lwt = ''; }                 # LWT Topic setzen (bei Update)
  if (!defined $mqtt_upt) { $mqtt_upt = ''; }                 # Uptime Topic setzen (bei Update)
  if (!defined $mqtt_rssi) { $mqtt_rssi = ''; }               # RSSI Topic setzen (bei Update)
  if (!defined $mqtt_load) { $mqtt_load = ''; }               # Load Topic setzen (bei Update)
  chomp($mqtt_user);                                          # Zeilenendezeichen vom Username entfernen
  chomp($mqtt_pass);                                          # Zeilenendezeichen vom Passwort entfernen
  chomp($mqtt_lwt);                                           # Zeilenendezeichen des LWT Topic entfernen
  chomp($mqtt_upt);                                           # Zeilenendezeichen des Uptime Topic entfernen
  chomp($mqtt_rssi);                                          # Zeilenendezeichen des RSSI Topic entfernen
  chomp($mqtt_load);                                          # Zeilenendezeichen des Load Topic entfernen

  if ($mqtt_use !~ /^[0-1]$/) {                               # wenn Aktivierung nicht plausibel
    $mqtt_use = 0;                                            # MQTT ausschalten
    $mqtt_addr = 'localhost';                                 # MQTT-Server-Adresse voreinstellen
    $mqtt_port = '1883';                                      # MQTT-Server-Port voreinstellen
    $mqtt_vali = 2;                                           # MQTT-Gültigkeitsdauer voreinstellen
    $mqtt_user = '';                                          # Username für MQTT-Server voreinstellen
    $mqtt_pass = '';                                          # Passwort für MQTT-Server voreinstellen
    $mqtt_lwt = 'Haus/Technik/Matrixuhr-1/LWT';               # LWT Topic voreinstellen
    $mqtt_upt = 'Haus/Technik/Matrixuhr-1/uptime';            # Uptime Topic voreinstellen
    $mqtt_rssi = 'Haus/Technik/Matrixuhr-1/rssi';             # RSSI Topic voreinstellen
    $mqtt_load = 'Haus/Technik/Matrixuhr-1/load';             # Load Topic voreinstellen
  }
  if (!defined $mqtt_vali) { $mqtt_vali = 2; }                # MQTT-Gültigkeitsdauer voreinstellen (nach Update)
}
else {                                                        # wenn nicht alle 3 Basis-Parameter vorhanden
  $mqtt_use = 0;                                              # MQTT ausschalten
  $mqtt_addr = 'localhost';                                   # MQTT-Server-Adresse voreinstellen
  $mqtt_port = '1883';                                        # MQTT-Server-Port voreinstellen
  $mqtt_vali = 2;                                             # MQTT-Server-Gültigkeitsdauer voreinstellen
  $mqtt_user = '';                                            # Username für MQTT-Server voreinstellen
  $mqtt_pass = '';                                            # Passwort für MQTT-Server voreinstellen
  $mqtt_lwt = 'Haus/Technik/Matrixuhr-1/LWT';                 # LWT Topic voreinstellen
  $mqtt_upt = 'Haus/Technik/Matrixuhr-1/uptime';              # Uptime Topic voreinstellen
  $mqtt_rssi = 'Haus/Technik/Matrixuhr-1/rssi';               # RSSI Topic voreinstellen
  $mqtt_load = 'Haus/Technik/Matrixuhr-1/load';               # Load Topic voreinstellen
}
if ($test) {
  print "MQTT-Konfiguration ($mqtt_use, $mqtt_addr, $mqtt_port, $mqtt_vali, $mqtt_user, $mqtt_pass)\n";
  print "MQTT LWT ($mqtt_lwt)\n";
  print "MQTT Uptime ($mqtt_upt)\n";
  print "MQTT RSSI ($mqtt_rssi)\n";
  print "MQTT Load ($mqtt_load)\n";
}
                                                              # Daten einlesen: Zeitsynchronisierung
if (open($CFG, "<$cfgdir/$timename")) {                       # wenn Zeitsynchronisierungsdatei vorhanden -> öffnen
  $time_use = <$CFG>;                                         # Synchronisierungs-Aktivierung lesen
  $time_int = <$CFG>;                                         # Synchronisierungs-Intervall lesen
  close($CFG);                                                # Zeitsynchronisierungsdatei schließen
}
if ((defined $time_use) && (defined $time_int)) {             # wenn Aktivierung und Intervall vorhanden
  chomp($time_use);                                           # Zeilenendezeichen der Aktivierung entfernen
  chomp($time_int);                                           # Zeilenendezeichen des Intervalls entfernen
  if (($time_use !~ /^[0-1]$/) || ($time_int !~ /^\d+$/)) {   # wenn Aktivierung oder Intervall nicht plausibel
    $time_use = 0;                                            # Synchronisierung ausschalten
    $time_int = 60;                                           # Intervall auf 60 setzen
  }
  if (($time_int < 1) || ($time_int > 60)) {                  # wenn Intervall außerhalb des zulässigen Bereichs
    $time_int = 60;                                           # Intervall auf 60 setzen
  }
}
else {                                                        # wenn Aktivierung oder Intervall nicht vorhanden
  $time_use = 0;                                              # Synchronisierung ausschalten
  $time_int = 60;                                             # Intervall auf 60 setzen
}
if ($test) { print "Zeitsynchronisierung ($time_use, $time_int)\n"; }

$sencnt = 0;                                                  # Daten einlesen: Sensorlistendatei, Zähler zurücksetzen
if (open($CFG, "<$cfgdir/$senlname")) {                       # wenn Sensorlistendatei vorhanden -> öffnen
  for ($i = 0; $i < 12; $i ++) {                              # Schleife für alle Elemente der Sensorliste
    if (!eof($CFG)) {                                         # wenn kein Dateiende
      $temp = <$CFG>;                                         # Zeile lesen (MQTT-Topic und Datenquelle)
      chomp($temp);                                           # Zeilenendezeichen entfernen
      $j = index($temp, ' ');                                 # Leerzeichen suchen
      if ($j >= 0) {                                          # wenn Leerzeichen gefunden
        $topic = substr($temp, 0, $j);                        # MQTT-Topic aus Zeile holen
        $format = substr($temp, $j + 1, 1);                   # Datenformat aus Zeile holen
      }
      else {                                                  # wenn kein Leerzeichen gefunden
        $topic = $temp;                                       # Zeile enthält nur MQTT-Topic
        $format = 0;                                          # Datenformat = 0 setzen (0 Nachkommastellen)
      }
      $sencnt ++;                                             # Zähler erhöhen
    }
    else {                                                    # wenn Dateiende
      $topic = '';                                            # leeres MQTT-Topic setzen
      $format = 0;                                            # Datenformat = 0 setzen
    }
    push @sentlist, $topic;                                   # MQTT-Topic an Liste anfügen
    push @senflist, $format;                                  # Datenformat an Liste anfügen
  }
  close($CFG);                                                # Sensorlistendatei schließen
}
else {                                                        # wenn Sensorlistendatei nicht vorhanden
  for ($i = 0; $i < 12; $i ++) {                              # Schleife für alle Elemente der Sensorliste
    push @sentlist, '';                                       # leere Zeile setzen
    push @senflist, 0;
  }
}
if ($test) { print "Sensorliste ($sencnt)\n"; }
                                                              # Daten einlesen: Sensoren
$senmax = 0;                                                  # Anzahl der auszugebenden Sensordatensätze auf 0 setzen
if (open($CFG, "<$cfgdir/$senname")) {                        # wenn Sensorliste vorhanden -> öffnen
  while (!eof($CFG)) {                                        # solange Dateiende noch nicht erreicht ist
    $temp = <$CFG>;                                           # eine Zeile lesen
    chomp($temp);                                             # Zeilenendezeichen entfernen
    push @sensors, $temp;                                     # an Liste anfügen
    $senmax ++;                                               # Anzahl der Sensoren erhöhen
  }
  close($CFG);                                                # Sensorliste schließen
}
if ($test) { print "Sensoren ($senmax)\n"; }

$sencnt = 0;                                                  # Daten einlesen: Alarme, Zähler zurücksetzen
for ($i = 1; $i < 9; $i ++) {                                 # Konfiguration der 8 Matrixuhr-Alarme lesen
  if (open($CFG, "<$cfgdir/$almname$i")) {                    # wenn Konfigurationsdatei vorhanden -> öffnen
    $temp = <$CFG>;                                           # Zeile 1 lesen (Alarm-Parameter)
    if (defined $temp) { chomp($temp); }                      # wenn Parameter gelesen -> Zeilenendezeichen entfernen
    else { $temp = ''; }                                      # sonst leere Zeile setzen
    $topic = <$CFG>;                                          # Zeile 2 lesen (MQTT-Topic oder Bedingung)
    if (defined $topic) {                                     # wenn Topic oder Bedingung gelesen
      chomp($topic);                                          # Zeilenendezeichen entfernen
      $sencnt ++;                                             # Zähler erhöhen
    }
    else { $topic = ''; }                                     # sonst leere Zeile setzen
    close($CFG);                                              # Konfigurationsdatei schließen
  }
  else {                                                      # wenn Konfigurationsdatei nicht vorhanden
    $temp = '';                                               # leeren Alarmparameter setzen
    $topic = '';                                              # leeres Topic oder leere Bedingung setzen
  }
  push @almparam, $temp;                                      # an Alarm-Parameter-Liste anfügen
  push @almtlist, $topic;                                     # an Topic- bzw. Bedingungsliste anfügen
  push @almtflag, 0;                                          # 0 an Timer-Flag-Liste anfügen
}
if ($test) { print "Alarme ($sencnt)\n"; }
                                                              # Daten einlesen: Geburtstage
if (open($CFG, "<$cfgdir/$bdname")) {                         # wenn Geburtstagsliste vorhanden -> öffnen
  while (!eof($CFG)) {                                        # solange Dateiende noch nicht erreicht ist
    $temp = <$CFG>;                                           # eine Zeile lesen
    chomp($temp);                                             # Zeilenendezeichen entfernen
    push @bdays, $temp;                                       # an Liste anfügen
  }
  close($CFG);                                                # Geburtstagsliste schließen
}
if ($test) { print "Geburtstage (", scalar @bdays, ")\n"; }
                                                              # Daten einlesen: Anruf-Konfiguration
if (open($CFG, "<$cfgdir/$callname")) {                       # wenn Anruf-Konfigurationsdatei vorhanden -> öffnen
  $call_use = <$CFG>;                                         # Aktivierung lesen
  $call_numb = <$CFG>;                                        # eigene Nummer lesen
  $call_evnt = <$CFG>;                                        # MQTT-Topic für Ereignis lesen
  $call_name = <$CFG>;                                        # MQTT-Topic für Name lesen
  $call_extn = <$CFG>;                                        # MQTT-Topic für Externe Nummer lesen
  $call_intn = <$CFG>;                                        # MQTT-Topic für Interne Nummer lesen
  close($CFG);                                                # Anruf-Konfigurationsdatei schließen
}
if ((defined $call_use) && (defined $call_numb) && (defined $call_evnt) &&
    (defined $call_name) && (defined $call_extn) && (defined $call_intn)) { # wenn alle 6 Parameter vorhanden
  chomp($call_use);                                           # Zeilenendezeichen der Aktivierung entfernen
  chomp($call_numb);                                          # Zeilenendezeichen der eigenen Nummer entfernen
  chomp($call_evnt);                                          # Zeilenendezeichen des Ereignis-Topic entfernen
  chomp($call_name);                                          # Zeilenendezeichen des Namen-Topic entfernen
  chomp($call_extn);                                          # Zeilenendezeichen der Externe-Nummer-Topic entfernen
  chomp($call_intn);                                          # Zeilenendezeichen der Interne-Nummer-Topic entfernen
  if (($call_use !~ /^[0-8]$/) || ($call_numb !~ /^\d*$/)) {  # wenn Aktivierung oder eigene Nummer nicht plausibel
    $call_use = 0;                                            # Anrufeanzeige ausschalten
    $call_numb = '';                                          # eigene Nummer löschen
    $call_evnt = '';                                          # Ereignis-Topic löschen
    $call_name = '';                                          # Name-Topic löschen
    $call_extn = '';                                          # Externe-Nummer-Topic löschen
    $call_intn = '';                                          # Interne-Nummer-Topic löschen
  }
}
else {                                                        # wenn nicht alle 6 Parameter vorhanden
  $call_use = 0;                                              # Anrufeanzeige ausschalten
  $call_numb = '';                                            # eigene Nummer löschen
  $call_evnt = '';                                            # Ereignis-Topic löschen
  $call_name = '';                                            # Name-Topic löschen
  $call_extn = '';                                            # Externe-Nummer-Topic löschen
  $call_intn = '';                                            # Interne-Nummer-Topic löschen
}
if ($test and $call_use > 0) {
  print "Anrufe, eigene Nummer ($call_numb)\n";
  print "Anrufe, MQTT Topic Ereignis ($call_evnt)\n";
  print "Anrufe, MQTT Topic Name ($call_name)\n";
  print "Anrufe, MQTT Topic Extern ($call_extn)\n";
  print "Anrufe, MQTT Topic ($call_intn)\n";
}
                                                              # Daten einlesen: Wetter-Konfiguration
if (open($CFG, "<$cfgdir/$wename")) {                         # wenn Wetter-Konfigurationsdatei vorhanden -> öffnen
  $we_use = <$CFG>;                                           # Aktivierung lesen
  $we_lng = <$CFG>;                                           # APPID lesen (nicht genutzt)
  $we_lng = <$CFG>;                                           # Ortsname lesen (nicht genutzt)
  $we_lng = <$CFG>;                                           # Breitengrad lesen (nicht genutzt)
  $we_lng = <$CFG>;                                           # Längengrad lesen (nicht genutzt)
  $we_lng = <$CFG>;                                           # Sprache lesen
  close($CFG);                                                # Wetter-Konfigurationsdatei schließen
}
if ((defined $we_use) && (defined $we_lng)) {                 # wenn Aktivierung und Sprache vorhanden
  chomp($we_use);                                             # Zeilenendezeichen der Aktivierung entfernen
  chomp($we_lng);                                             # Zeilenendezeichen der Sprache entfernen
  if ($we_use !~ /^[0-1]$/) {                                 # wenn Aktivierung nicht plausibel
    $we_use = 0;                                              # Wetterabfrage ausschalten
    $we_lng = 'de';                                           # Sprache auf Deutsch setzen
  }
}
else {                                                        # wenn nicht alle Parameter vorhanden
  $we_use = 0;                                                # Wetterabfrage ausschalten
  $we_lng = 'de';                                             # Sprache auf Deutsch setzen
}
$sencnt = 0;                                                  # Zähler zurücksetzen
$temp = index $wcname, '??';                                  # Position der Sprachkennung im Dateinamen ermitteln
if ($temp >= 0) {                                             # wenn Position gefunden
  $wcname = substr($wcname, 0, $temp) . $we_lng . substr($wcname, $temp + 2); # Sprachkennung in Dateiname einfügen
  if (open($CFG, "<$cfgdir/$wcname")) {                       # wenn Wetterlage-Texte-Datei vorhanden -> öffnen
    while (!eof($CFG)) {                                      # solange Dateiende noch nicht erreicht ist
      $temp = <$CFG>;                                         # eine Zeile lesen
      chomp($temp);                                           # Zeilenendezeichen entfernen
      $temp = decode('UTF-8', $temp, Encode::FB_DEFAULT);     # in Perl-Format wandeln
      push @wecndn, encode('UTF-8', substr($temp, 0, 3), Encode::FB_DEFAULT); # Wetterlagenummer wandeln und speichern
      push @wecndt, encode('UTF-8', substr($temp, 4), Encode::FB_DEFAULT); # Wetterlagetext in UTF-8 wandeln und spei.
      $sencnt ++;                                             # Zähler erhöhen
    }
    close($CFG);                                              # Wetterlage-Datei schließen
  }
}
$sencn2 = 0;                                                  # Zähler zurücksetzen
if (open($CFG, "<$cfgdir/$icname")) {                         # wenn Icondatei vorhanden -> öffnen
  while (!eof($CFG)) {                                        # solange Dateiende noch nicht erreicht ist
    $temp = <$CFG>;                                           # eine Zeile lesen
    chomp($temp);                                             # Zeilenendezeichen entfernen
    push @icons, $temp;                                       # Iconname speichern
    $sencn2 ++;                                               # Zähler erhöhen
  }
  close($CFG);
}
if ($test) { print "Wetterdaten ($we_use, $we_lng, $sencnt, $sencn2)\n"; }

if (open($CFG, "<$cfgdir/$sdname")) {                         # wenn spezielle MQTT-Konfigurationsdatei vorh. -> öffnen
  $sdtop1 = <$CFG>;                                           # MQTT Topic für Wetterdaten lesen
  $sdtop2 = <$CFG>;                                           # MQTT Topic für Kraftstoffdaten lesen
  $sdtop3 = <$CFG>;                                           # MQTT Topic für Nachrichten lesen
  close($CFG);                                                # Wetter-Konfigurationsdatei schließen
  chomp($sdtop1);                                             # Zeilenendezeichen entfernen
  chomp($sdtop2);                                             # Zeilenendezeichen entfernen
  if (!defined $sdtop3) { $sdtop3 = ''; }                     # MQTT Topic für Nachrichten setzen (bei Update)
  chomp($sdtop3);                                             # Zeilenendezeichen entfernen
}
else {                                                        # wenn spezielle Konfigurationsdatei nicht vorhanden
  $sdtop1 = '';                                               # MQTT Topic für Wetterdaten löschen
  $sdtop2 = '';                                               # MQTT Topic für Kraftstoffdaten löschen
  $sdtop3 = '';                                               # MQTT Topic für Nachrichten löschen
}
if ($test) {
  print "MQTT Topic Wetter ($sdtop1)\n";
  print "MQTT Topic Kraftstoffdaten ($sdtop2)\n";
  print "MQTT Topic Nachrichten ($sdtop3)\n";
}

# Start, MQTT aktivieren

$sencnt = 0;                                                  # Zähler zurücksetzen
if ($mqtt_use) {                                              # wenn MQTT eingeschaltet
  $ENV{MQTT_SIMPLE_ALLOW_INSECURE_LOGIN} = 1;                 # unverschlüsselte Verbindungen erlauben
  $mqtt = Net::MQTT::Simple->new("$mqtt_addr:$mqtt_port");    # Verbindung zum MQTT-Server herstellen
  if ($mqtt_user gt '' and $mqtt_pass gt '') {                # wenn MQTT Username und Passwort vorhanden
    $mqtt->login($mqtt_user, $mqtt_pass);                     # Username und Passwort setzen
  }
  if ($mqtt_lwt gt '') {                                      # wenn LWT Topic vorhanden
    $mqtt->last_will($mqtt_lwt, $mqtt_lmsg, 1);               # LWT-Nachricht hinterlegen
    $mqtt->retain($mqtt_lwt, $mqtt_smsg);                     # LWT-Startnachricht senden
  }
  if ($sentlist[0] gt '') { $mqtt->subscribe($sentlist[0], \&sensorA); $sencnt ++; }   # Sensor A abonnieren
  if ($sentlist[1] gt '') { $mqtt->subscribe($sentlist[1], \&sensorB); $sencnt ++; }   # Sensor B abonnieren
  if ($sentlist[2] gt '') { $mqtt->subscribe($sentlist[2], \&sensorC); $sencnt ++; }   # Sensor C abonnieren
  if ($sentlist[3] gt '') { $mqtt->subscribe($sentlist[3], \&sensorD); $sencnt ++; }   # Sensor D abonnieren
  if ($sentlist[4] gt '') { $mqtt->subscribe($sentlist[4], \&sensorE); $sencnt ++; }   # Sensor E abonnieren
  if ($sentlist[5] gt '') { $mqtt->subscribe($sentlist[5], \&sensorF); $sencnt ++; }   # Sensor F abonnieren
  if ($sentlist[6] gt '') { $mqtt->subscribe($sentlist[6], \&sensorG); $sencnt ++; }   # Sensor G abonnieren
  if ($sentlist[7] gt '') { $mqtt->subscribe($sentlist[7], \&sensorH); $sencnt ++; }   # Sensor H abonnieren
  if ($sentlist[8] gt '') { $mqtt->subscribe($sentlist[8], \&sensorI); $sencnt ++; }   # Sensor I abonnieren
  if ($sentlist[9] gt '') { $mqtt->subscribe($sentlist[9], \&sensorJ); $sencnt ++; }   # Sensor J abonnieren
  if ($sentlist[10] gt '') { $mqtt->subscribe($sentlist[10], \&sensorK); $sencnt ++; } # Sensor K abonnieren
  if ($sentlist[11] gt '') { $mqtt->subscribe($sentlist[11], \&sensorL); $sencnt ++; } # Sensor L abonnieren
  if ($almtlist[0] gt '') { $mqtt->subscribe($almtlist[0], \&alarm1); $sencnt ++; }    # Alarm 1 abonnieren
  if ($almtlist[1] gt '') { $mqtt->subscribe($almtlist[1], \&alarm2); $sencnt ++; }    # Alarm 2 abonnieren
  if ($almtlist[2] gt '') { $mqtt->subscribe($almtlist[2], \&alarm3); $sencnt ++; }    # Alarm 3 abonnieren
  if ($almtlist[3] gt '') { $mqtt->subscribe($almtlist[3], \&alarm4); $sencnt ++; }    # Alarm 4 abonnieren

  if ($call_use > 0) {                                                                 # wenn Anrufanzeige aktiviert
    if ($call_evnt gt '') { $mqtt->subscribe($call_evnt, \&call1); $sencnt ++; }       # Ereignis abonnieren
    if ($call_name gt '') { $mqtt->subscribe($call_name, \&call2); $sencnt ++; }       # Name abonnieren
    if ($call_extn gt '') { $mqtt->subscribe($call_extn, \&call3); $sencnt ++; }       # Externe Nummer abonnieren
    if ($call_intn gt '') { $mqtt->subscribe($call_intn, \&call4); $sencnt ++; }       # Interne Nummer abonnieren
    $call_evnt = '';                                          # Topic löschen, dient jetzt als Variable Ereignis
    $call_name = '';                                          # Topic löschen, dient jetzt als Variable Name
    $call_extn = '';                                          # Topic löschen, dient jetzt als Variable Ext. Nummer
    $call_intn = '';                                          # Topic löschen, dient jetzt als Variable Int. Nummer
  }
  if ($sdtop1 gt '') { $mqtt->subscribe($sdtop1, \&sdtopic1); } # Wetterdaten abonnieren
  if ($sdtop2 gt '') { $mqtt->subscribe($sdtop2, \&sdtopic2); } # Kraftstoffdaten abonnieren
  if ($sdtop3 gt '') { $mqtt->subscribe($sdtop3, \&sdtopic3); } # Nachrichten abonnieren
}
if ($test) { print "MQTT-Abonnements ($sencnt)\n"; }
                                                              # wenn Verzeichnis für flüchtige Daten nicht existiert
if (!(-d $sendir)) { $sendir = $cfgdir; }                     # Konfigurationsverzeichnis verwenden (1-Wire, Wetter)
if (-e "$sendir/$jsname") { unlink "$sendir/$jsname"; }       # gegebenenfalls alte Wetterdatei löschen
if ($test) { print "Verzeichnis für 1-Wire-Sensor und Wetterdaten: $sendir\n"; }

# Start, serielle Schnittstelle aktivieren, Startwerte setzen

system("stty -F $senif 9600 -echo");                          # Baudrate einstellen und Echo ausschalten
open($SER, ">$senif") or die $!;                              # serielle Schnittstelle zum Schreiben öffnen
print $SER "\n\n";                                            # 2x Zeilenende senden (Empfängersynchronisierung)
$lasttime = 0;                                                # Zeitstempel für Datenausgabe setzen
$we_time = 0;                                                 # Zeitstempel für letzten Wetterdaten-Empfang setzen
$mqttftim = 0;                                                # Zeitstempel für letzten Kraftstoffdaten-Empfang setzen
$mqttwval = '{}';                                             # Wetterdaten initialisieren (leeres JSON-Objekt)
$mqttfval = '[]';                                             # Kraftstoffdaten initialisieren (leeres JSON-Array)
$lastsync = time - $time_int * 60 + 60;                       # Zeitstempel für Zeitsynchronisierung setzen (1 Minute)
$count = 1;                                                   # Datenzähler auf Anfangswert setzen
$sencnt = 1;                                                  # Sensornummerzähler auf Anfangswert setzen
$laststat = time;                                             # Zeistempel für letzte MQTT-Statusmeldung setzen

for ($i = 1; $i < 32; $i ++) {                                # 31 Datensätze ausgeben
  $sendat = $i;                                               # Sensorkennung entspricht Zähler
  if ($sendat < 10) { $sendat = '0' . $sendat; }              # wenn einstellig -> 0 voranstellen
  print $SER "$sendat:\n";                                    # Leeres Sensorfeld ausgeben
}
for ($i = 1; $i < 9; $i ++) { print $SER "A$i:0\n"; }         # 8 Alarme ausschalten
if ($test) { print "Datenfelder und Alarme zurücksetzen\n"; }
if ($test) { print "Programmschleife...\n"; }

# Endlosschleife zur zyklischen Ausgabe von Sensordaten, Geburtstagen, spezieller Alarme und Zeitsynchronisierung

while (1) {                                                   # Endlosschleife

  if ($mqtt_use) { $mqtt->tick(0.1); }                        # wenn MQTT aktiv -> empfangene MQTT-Nachrichten abfragen
  else { sleep 1; }                                           # wenn nicht -> 1 Sekunde warten

  if (time > $lasttime + 1) {                                 # wenn Zeit für nächste Datenausgabe erreicht

                                                              # Generieren und Senden eines Sensordatensatzes
    if ($count < $senmax + 1) {                               # wenn Zeitpunkt für Sensorausgabe
      $seline = $sensors[$count - 1];                         # Sensordatenzeile aus Liste holen
      if ($seline =~ /\[X\]/) {                               # wenn Wetterlagetext im Datensatz enthalten
        if (defined $we_vals[11]) {                           # wenn Wetterlagetext vorhanden
          if (time < $we_time + 3700) {                       # wenn Wetterlagetext nicht älter als 1 Stunde
            @we_str = split / /, $we_vals[11];                # Wetterlagetext in einzelne Wörter zerlegen
            $i = 0;                                           # Wortzähler löschen
            do {                                              # Schleife
              $sendat = $sencnt;                              # Sensorkennung vom Sensornummerzähler übernehmen
              if ($sendat < 10) { $sendat = '0' . $sendat }   # wenn einstellig -> 0 voranstellen
              $sendat .= ":$we_str[$i]";                      # Doppelpunkt und Wort ergänzen
              if ($i < @we_str - 1) {                         # wenn weitere Wörter vorhanden
                if (length($we_str[$i]) + length($we_str[$i + 1]) < 11) { # wenn nächstes Wort auf die Zeile passt
                  $i ++;                                      # Wortzähler erhöhen
                  $sendat .= " $we_str[$i]";                  # nächstes Wort ergänzen
                }
              }
              print $SER "$sendat\n";                         # Datensatz senden
              if ($test) { print "$sendat\n"; }
              $sencnt ++;                                     # Sensornummerzähler erhöhen
              $i ++;                                          # Wortzähler erhöhen
            } while ($i < @we_str);                           # wiederholen bis alle Wörter bearbeitet sind
          }
        }
      }
      else {                                                  # wenn kein Wetterlagetext -> Zeile zeichenweise lesen
        $j = 0;                                               # Index auf erstes Zeichen der Sensorzeile setzen
        $semode = 0;                                          # Sensormodus normal setzen
        $sendat = $sencnt;                                    # Sensorkennung vom Sensornummerzähler übernehmen
        if ($sendat < 10) { $sendat = '0' . $sendat }         # wenn einstellig -> 0 voranstellen
        $sendat .= ':';                                       # Doppelpunkt ergänzen
        do {                                                  # Schleife (Sensordatenzeile zeichenweise auswerten)
          if ($semode == 1) {                                 # wenn Sensornamenmodus
            if (substr($seline, $j, 1) eq ']') {              # wenn Zeichen "]"
              $semode = 0;                                    # Sensormodus normal setzen
              $senok = 0;                                     # Sensorwert zunächst nicht vorhanden
              if ($sename =~ /^[A-L]$/) {                     # wenn Sensorname im Bereich A-L (Sensorwert über MQTT)
                $senval = $senvals[ord($sename) - ord('A')];  # Sensorwert aus Liste holen
                $setime = $sentimes[ord($sename) - ord('A')]; # Zeitstempel aus Liste holen
                if ((defined $senval) && (defined $setime)) { # wenn Sensorwert und Zeitstempel vorhanden
                  if (time < $setime + $mqtt_vali * 60) {     # wenn Sensorwert innerhalb der Gültigkeitsdauer
                    $senok = 1;                               # Sensorwert ist in Ordnung
                    $sendat .= $senval;                       # Sensorwert ergänzen
                  }
                }
              }
              if ($sename =~ /^[M-W]$/) {                     # wenn Sensorname im Bereich M-U (Wetterdaten, Symbole)
                $senval = $we_vals[ord($sename) - ord('M')];  # Sensorwert aus Liste holen
                if (defined $senval) {                        # wenn Sensorwert vorhanden
                  if (time < $we_time + 3700) {               # wenn Sensorwert nicht älter als 1 Stunde
                    $senok = 1;                               # Sensorwert ist in Ordnung
                    $sendat .= $senval;                       # Sensorwert ergänzen
                  }
                }
              }
              if ($sename =~ /^[Y]$/) {                       # wenn Sensorname Y (Sonnenauf/untergang)
                if (defined $we_vals[12]) {                   # wenn Sonnendaten vorhanden
                  if (time < $we_time + 3700) {               # wenn Sonnendaten nicht älter als 1 Stunde
                    $senok = 1;                               # Sonnendaten sind in Ordnung
                    $sendat .= $we_vals[12];                  # Sonnendaten ergänzen
                  }
                }
              }
              if ($sename =~ /^[Z]$/) {                       # wenn Sensorname Z (1-Wire-Sensor)
                if (defined $w1_val) {                        # wenn Sensorwert vorhanden
                  $senok = 1;                                 # Sensorwert ist in Ordnung
                  $sendat .= $w1_val;                         # Sensorwert ergänzen
                }
              }
              if ($sename =~ /^[a-d]$/) {                     # wenn Sensorname im Bereich a-d (Kraftstoffpreise)
                $senval = $fu_vals[ord($sename) - ord('a')];  # Preis aus Liste holen
                if (defined $senval) {                        # wenn Preis vorhanden
                  if (time < $mqttftim + 3700) {              # wenn Preis nicht älter als 1 Stunde
                    $senok = 1;                               # Sensorwert ist in Ordnung
                    $sendat .= $senval;                       # Sensorwert ergänzen
                  }
                }
              }
              if ($senok == 0) { $sendat .= "---"; }          # wenn kein Sensorwert -> Ersatzzeichen ergänzen
            }
            else {                                            # wenn kein Zeichen "]"
              $sename = substr($seline, $j, 1);               # Sensorname übernehmen
            }
          }
          else {                                              # wenn Normalmodus
            if (substr($seline, $j, 1) eq '[') {              # wenn Zeichen "["
              $semode = 1;                                    # Sensornamenmodus setzen
              $sename = '@';                                  # Sensorname löschen
            }
            else {                                            # wenn kein Zeichen "["
              $sendat .= substr($seline, $j, 1);              # Zeichen ungefiltert ergänzen
            }
          }
          $j ++;                                              # nächstes Zeichen
        } while ($j < length($seline));                       # Schleife bis alle Zeichen bearbeitet
        if (!($sendat =~ /\-{3}/)) {                          # wenn kein Ersatzzeichen im Datensatz
          print $SER "$sendat\n";                             # Datensatz senden
          if ($test) { print "$sendat\n"; }
        }
        $sencnt ++;                                           # Sensornummerzähler erhöhen
      }
    }
                                                              # Senden aller aktiven Geburtstage
    if ($count == $senmax + 1) {                              # wenn Zeitpunkt für Geburtstagsausgabe
      $bdnumb = $sencnt;                                      # Sensorzählernummer entspricht Geburtstagsnummer
      ($sec, $min, $hou, $day, $mon, $yea, $wday, $yday, $dst) = localtime(time); # aktuelle Zeit lesen
      if ($day < 10) { $day = '0' . $day; }                   # wenn Tag einstellig -> 0 voranstellen
      $mon += 1;                                              # Monat korrigieren (0-11 -> 1-12)
      if ($mon < 10) { $mon = '0' . $mon; }                   # wenn Monat einstellig -> 0 voranstellen
      $yea += 1900;                                           # Jahr korrigieren
      $bdayflag = 0;                                          # Geburtstags-Flag löschen
      for ($i = 0; $i < @bdays; $i ++) {                      # Geburtstagsliste durchsuchen
        $bdline = $bdays[$i];                                 # eine Zeile lesen
        if ((substr($bdline, 4, 2) eq $mon) && (substr($bdline, 6, 2) eq $day)) { # wenn Monat und Tag identisch
          $sendat = $bdnumb;                                  # Geburtstage werden als Sensorwert ausgegeben
          if ($sendat < 10) { $sendat = '0' . $sendat; }      # wenn einstellig -> 0 voranstellen
          print $SER $sendat . ':' . substr($bdline, 9) . ' ' . ($yea - substr($bdline, 0, 4)) . "\n";
                                                              # Kennung, Name und berechnetes Alter ausgeben
          if ($test) { print $sendat . ':' . substr($bdline, 9) . ' ' . ($yea - substr($bdline, 0, 4)) . "\n"; }
          $bdnumb ++;                                         # Kennung erhöhen
          $bdayflag = 1;                                      # Geburtstags-Flag setzen
        }
      }                                                       # Abfrage der Daten des 1-Wire-Sensors
      undef $w1_val;                                          # zunächst 1-Wire-Temperaturwert löschen
      if (open($SEN, "<$sendir/$w1name")) {                   # Sensorwertdatei öffnen
        $w1_val = <$SEN>;                                     # Sensorwert lesen
        close($SEN);                                          # Sensorwertdatei schließen
        chomp($w1_val);                                       # Zeilenendezeichen entfernen
        if ((time - (stat "$sendir/$w1name")[9]) > 120) {     # wenn Wert älter als 2 Minuten
          undef $w1_val;                                      # ungültig machen
        }
      }
      for ($i = $bdnumb; $i < $bdnumb + 4; $i ++) {           # Löschen von 4 nachfolgenden Datenfeldern
        $sendat = $i;                                         # Daten werden als Sensorwert ausgegeben
        if ($sendat < 10) { $sendat = '0' . $sendat; }        # wenn einstellig -> 0 voranstellen
        print $SER $sendat . ":\n";                           # Doppelpunkt ergänzen und ausgeben
        if ($test) { print $sendat . ":\n"; }
      }
    }
                                                              # Auswerten von speziellen Alarmen und Status senden
    if ($count == $senmax + 2) {                              # wenn Zeitpunkt für spezielle Alarme
      for ($i = 5; $i < 9; $i ++) {                           # Alarme 5-8 bearbeiten
        $alline = $almtlist[$i - 1];                          # Bedingung aus Liste holen
        if (defined $alline) {                                # wenn Bedingung vorhanden
          $sename = '@';                                      # Sensorname löschen
          $j = 0;                                             # Position des Vergleichsoperators löschen
          if (substr($alline, 0, 1) eq '[') {                 # wenn Zeile mit "[" beginnt
            $sename = substr($alline, 1, 1);                  # Sensorname ermitteln
            $j = 3;                                           # Position des Vergleichsoperators setzen
            $senok = 0;                                       # Sensorwert zunächst nicht vorhanden
            if ($sename =~ /^[A-L]$/) {                       # wenn Sensorname im Bereich A-L
              $senval = $senvals[ord($sename) - ord('A')];    # Sensorwert aus Liste holen
              $setime = $sentimes[ord($sename) - ord('A')];   # Zeitstempel aus Liste holen
              if ((defined $senval) && (defined $setime)) {   # wenn Sensorwert und Zeitstempel vorhanden
                if (time < $setime + $mqtt_vali * 60) {       # wenn Sensorwert innerhalb der Gültigkeitsdauer
                  $senok = 1;                                 # Wert ok
                }
              }
            }
            if ($sename =~ /^[M-V]$/) {                       # wenn Sensorname im Bereich M-V
              $senval = $we_vals[ord($sename) - ord('M')];    # Sensorwert aus Liste holen
              if (defined $senval) {                          # wenn Sensorwert vorhanden
                if (time < $we_time + 3700) { $senok = 1; }   # wenn Sensorwert nicht älter als 1 Stunde -> ok
              }
            }
            if ($sename =~ /^[a-d]$/) {                       # wenn Sensorname im Bereich a-d (Kraftstoffpreise)
              $senval = $fu_vals[ord($sename) - ord('a')];    # Preis aus Liste holen
              if (defined $senval) {                          # wenn Preis vorhanden
                if (time < $mqttftim + 3700) { $senok = 1; }  # wenn Preis nicht älter als 1 Stunde
              }
            }
            if ($sename =~ /^[Z]$/) {                         # wenn Sensorname Z
              $senval = $w1_val;                              # Sensorwert holen
              if (defined $senval) { $senok = 1; }            # wenn Sensorwert vorhanden -> ok
            }
            if ($senok) {                                     # wenn Sensorwert vorhanden
              undef $comval;                                  # Vergleichwert zunächst ungültig machen
              $comnam = '@';                                  # Vergleichssensorname löschen
              if (substr($alline, $j + 1, 1) eq '[') {        # wenn Vergleichswert ebenfalls ein Sensorname
                if (substr($alline, $j + 3, 1) eq ']') {      # wenn Sensorname mit "]" endet
                  $comnam = substr($alline, $j + 2, 1);       # Vergleichssensorname ermitteln
                }
                if ($comnam =~ /^[A-L]$/) {                   # wenn Sensorname im Bereich A-L
                  $comval = $senvals[ord($comnam) - ord('A')];  # Sensorwert aus Liste holen
                  $comtim = $sentimes[ord($comnam) - ord('A')]; # Zeitstempel aus Liste holen
                  if ((defined $comval) && (defined $comtim)) { # wenn Sensorwert und Zeitstempel vorhanden
                    if (time < $comtim + $mqtt_vali * 60) { } # wenn Sensorwert innerhalb der Gültigkeitsdauer
                    else { undef $comval; }                   # wenn nicht -> ungültig machen
                  }
                }
                if ($comnam =~ /^[M-V]$/) {                   # wenn Sensorname im Bereich M-V
                  $comval = $we_vals[ord($comnam) - ord('M')]; # Sensorwert aus Liste holen
                  if (defined $comval) {                      # wenn Sensorwert vorhanden
                    if (time < $we_time + 3700) { }           # wenn Sensorwert nicht älter als 1 Stunde
                    else { undef $comval; }                   # wenn nicht -> ungültig machen
                  }
                }
                if ($comnam =~ /^[Z]$/) {                     # wenn Sensorname Z
                  if (defined $w1_val) { $comval = $w1_val; } # wenn Sensorwert vorhanden -> Sensorwert holen
                }
                if ($comnam =~ /^[a-d]$/) {                   # wenn Sensorname im Bereich a-d (Kraftstoffpreise)
                  $comval = $fu_vals[ord($comnam) - ord('a')]; # Preis aus Liste holen
                  if (defined $comval) {                      # wenn Preis vorhanden
                    if (time < $mqttftim + 3700) { }          # wenn Preis nicht älter als 1 Stunde
                    else { undef $comval; }                   # wenn nicht -> ungültig machen
                  }
                }
              }
              else {                                          # wenn Vergleichswert direkt angegeben
                $comval = substr($alline, $j + 1);            # Vergleichswert lesen
              }
              if (defined $comval) {                          # wenn gültiger Vergleichswert vorhanden
                if (substr($alline, $j, 1) eq '<') {          # wenn Vergleichsoperator "<"
                  if ($senval < $comval) {                    # wenn Bedingung zutrifft (Sensorwert < Vergleichswert)
                    send_alarm($i, 1);                        # "Alarm ein" senden
                  }
                  else { send_alarm($i, 0) }                  # wenn Bedingung nicht zutrifft -> "Alarm aus" senden
                }
                if (substr($alline, $j, 1) eq '>') {          # wenn Vergleichsoperator ">"
                  if ($senval > $comval) {                    # wenn Bedingung zutrifft (Sensorwert > Vergleichswert)
                    send_alarm($i, 1);                        # "Alarm ein" Meldung senden
                  }
                  else { send_alarm($i, 0) }                  # wenn Bedingung nicht zutrifft -> "Alarm aus" senden
                }
              }
            }
          }
          if ($alline =~ /^[\+|\*|1|2|3|4|5|6|7]\:\d{2}\:\d{2}$/) { # wenn gültige Alarmzeit vorhanden
            ($sec, $min, $hou, $day, $mon, $yea, $wday, $yday, $dst) = localtime(time); # aktuelle Zeit lesen
            if ($hou == substr($alline, 2, 2)) {              # wenn Stunde erreicht
              if ($min == substr($alline, 5, 2)) {            # wenn Minute erreicht
                if ($wday == 0) { $wday = 7; }                # wenn Wochentag 0 -> auf 7 ändern (Sonntag)
                $almtflag[0] = 0;                             # Merker für Alarmauslösung löschen
                if ((substr($alline, 0, 1) eq '+') && $bdayflag) { # wenn Geburtstagsalarm und Geburtstags-Flag
                  $almtflag[0] = 1;                           # Merker setzen
                }
                elsif (substr($alline, 0, 1) eq $wday) {      # wenn Wochentagsalarm und passender Wochentag
                  $almtflag[0] = 1;                           # Merker setzen
                }
                elsif (substr($alline, 0, 1) eq '*') {        # wenn täglicher Alarm
                  $almtflag[0] = 1;                           # Merker setzen
                }
                if ($almtflag[0] && $almtflag[$i - 1] == 0) { # wenn Merker und Alarm-Timer-Flag = 0
                  send_alarm($i, 1);                          # "Alarm ein" senden
                  $almtflag[$i - 1] = 1;                      # Alarm-Timer-Flag = 1 setzen (neue Auslösung verhindern)
                }
              }
            }
            else { $almtflag[$i - 1] = 0; }                   # wenn Stunde nicht passt -> Alarm-Timer-Flag = 0 setzen
            send_alarm($i, 0);                                # "Alarm aus" senden
          }
        }
      }                                                       # bei ungenutzten Alarmen regelmäßige Meldung senden
      for ($i = 1; $i < 9; $i ++) {                           # Alarme 1-8 bearbeiten
        $alline = $almtlist[$i - 1];                          # Topic oder Bedingung aus Liste holen
        if ($alline eq '') {                                  # wenn kein Topic oder keine Bedingung vorhanden
          send_alarm($i, 0);                                  # "Alarm aus" senden
        }
      }
    }
                                                              # Senden der aktuellen Uhrzeit
    if ($count == $senmax + 3) {                              # wenn Zeitpunkt für Zeitsynchronisierung / Textnachricht
      if (($time_use) && (time > $lastsync + $time_int * 60)) { # wenn Synchronisierungs-Intervall erreicht
        $lastsync = time;                                     # neuen Zeitstempel setzen
        $curtim = time;                                       # aktuelle Zeit lesen
        ($sec, $min, $hou, $day, $mon, $yea, $wday, $yday, $dst) = localtime($curtim); # aktuelle Zeit lesen
        $sentim = sprintf ("%04d%02d%02d %02d:%02d:%02d.%02d %1d", $yea + 1900, $mon + 1,
          $day, $hou, $min, $sec, ($curtim - int($curtim)) * 100, $dst); # Datum und Zeit formatieren
        print $SER "TM:$sentim\n";                            # Zeit ausgeben
        if ($test) { print "TM:$sentim\n"; }
      }
      if (open($MES, "<$cfgdir/messagemu")) {                 # wenn Textnachricht vorhanden
        $meline = <$MES>;                                     # Textnachricht einlesen
        close($MES);                                          # Nachrichtendatei schließen
        if (defined $meline) {                                # wenn Textnachricht nicht leer
          print $SER "ME:$meline";                            # Textnachricht senden
          if ($test) { print "ME:$meline"; }
        }
        rename("$cfgdir/messagemu", "$cfgdir/messagemu.s");   # Nachrichtendatei umbenennen
      }
      if (defined $mqttmval) {                                # wenn eine Nachricht über MQTT empfangen wurde
        print $SER "ME:$mqttmval\n";                          # Textnachricht senden
        if ($test) { print "ME:$mqttmval\n"; }
        open($MES, ">$cfgdir/messagemu.s");                   # Nachrichtendatei anlegen
        print $MES "$mqttmval\n";                             # Nachricht schreiben
        close($MES);                                          # Nachrichtendatei schließen
        undef $mqttmval;                                      # Nachricht löschen
      }
    }
                                                              # Wetterdaten einlesen und speichern
    if ($count == $senmax + 4) {                              # wenn Zeitpunkt für Wetterdatenabfrage
      if ($sdtop1 gt '') {                                    # wenn MQTT Topic für Wetterdaten vorhanden
        if ($mqttwval gt '') {                                # wenn eine Datenpaket empfangen wurde
          $jsonbuf = decode_json($mqttwval);                  # JSON-Daten dekodieren und speichern
          undef $we_vals[0];                                  # Temperatur mit Nachkommastelle löschen
          undef $we_vals[1];                                  # Temperatur ohne Nachkommastelle löschen
          $temp = $jsonbuf->{'tempc'};                        # Temperaturwert ermitteln
          if (defined $temp) {                                # wenn Wert vorhanden
            if ($temp =~ /^\s*\-?\d*\.?\d*$/) {               # wenn plausibler Wert
              $we_vals[0] = sprintf("%.1f", $temp);           # Temperatur in °C mit einer Nachkommastelle speichern
              if (($we_vals[0] == 0) && ($we_vals[0] =~ /\-+/)) { # wenn Spezialfall -0 eintritt
                $temp = 0;                                    # Wert auf 0 setzen
                $we_vals[0] = sprintf("%.1f", $temp);         # und wiederum mit einer Nachkommastelle speichern
              }
              $we_vals[1] = sprintf("%.0f", $temp);           # Temperatur in °C ohne Nachkommastelle speichern
              if (($we_vals[1] == 0) && ($we_vals[1] =~ /\-+/)) { # wenn Spezialfall -0 eintritt
                $temp = 0;                                    # Wert auf 0 setzen
                $we_vals[1] = sprintf("%.0f", $temp);         # und wiederum ohne Nachkommastelle speichern
              }
            }
          }
          undef $we_vals[2];                                  # Luftfeuchtigkeit löschen
          $temp = $jsonbuf->{'humidity'};                     # Luftfeuchtigkeitswert ermitteln
          if (defined $temp) {                                # wenn Wert vorhanden
            if ($temp =~ /^\s*\-?\d*\.?\d*$/) {               # wenn plausibler Wert
              $we_vals[2] = sprintf("%.0f", $temp);           # Luftfeuchtigkeit in % ohne Nachkommastelle speichern
            }
          }
          undef $we_vals[3];                                  # Luftdruck löschen
          $temp = $jsonbuf->{'pressure'};                     # Luftdruck ermitteln
          if (defined $temp) {                                # wenn Wert vorhanden
            if ($temp =~ /^\s*\-?\d*\.?\d*$/) {               # wenn plausibler Wert
              $we_vals[3] = sprintf("%.0f", $temp);           # Luftdruck in hPa ohne Nachkommastelle speichern
            }
          }
          undef $we_vals[4];                                  # Windgeschwindigkeit in m/s löschen
          undef $we_vals[5];                                  # Windgeschwindigkeit in km/h löschen
          $temp = $jsonbuf->{'windspeed'};                    # Windgeschwindigkeit ermitteln
          if (defined $temp) {                                # wenn Wert vorhanden
            if ($temp =~ /^\s*\-?\d*\.?\d*$/) {               # wenn plausibler Wert
              $we_vals[4] = sprintf("%.1f", $temp);           # Windgeschwindigkeit in m/s mit einer Nachkommastelle
              $we_vals[5] = sprintf("%.0f", $temp * 3.6);     # Windgeschwindigkeit in km/h ohne Nachkommastelle
            }
          }
          undef $we_vals[6];                                  # Regenmenge in mm/h mit Nachkommastelle löschen
          undef $we_vals[7];                                  # Regenmenge in mm/h ohne Nachkommastelle löschen
          $temp = $jsonbuf->{'rain'};                         # Regenmenge ermitteln
          if (defined $temp) {                                # wenn Wert vorhanden
            if ($temp =~ /^\s*\-?\d*\.?\d*$/) {               # wenn plausibler Wert
              $we_vals[6] = sprintf("%.1f", $temp);           # Regenmenge in mm/h mit einer Nachkommastelle
              $we_vals[7] = sprintf("%.0f", $temp);           # Regenmenge in mm/h ohne Nachkommastelle
            }
          }
          undef $we_vals[8];                                  # Wolkendichte löschen
          $temp = $jsonbuf->{'clouds'};                       # Wolkendichte ermitteln
          if (defined $temp) {                                # wenn Wert vorhanden
            if ($temp =~ /^\s*\-?\d*\.?\d*$/) {               # wenn plausibler Wert
              $we_vals[8] = sprintf("%.0f", $temp);           # Wolkendichte in % ohne Nachkommastelle speichern
            }
          }
          $we_vals[9] = '$0';                                 # keine Windrichtung voreinstellen
          $temp = $jsonbuf->{'winddirection'};                # Windrichtung ermitteln
          if (defined $temp) {                                # wenn Wert vorhanden
            if ($temp =~ /^\s*\-?\d*\.?\d*$/) {               # wenn plausibler Wert
              if (($temp > 22.5) && ($temp <= 67.5)) { $we_vals[9] = '$6'; }      # Windrichtung SW
              elsif (($temp > 67.5) && ($temp <= 112.5)) { $we_vals[9] = '$7'; }  # Windrichtung W
              elsif (($temp > 112.5) && ($temp <= 157.5)) { $we_vals[9] = '$8'; } # Windrichtung NW
              elsif (($temp > 157.5) && ($temp <= 202.5)) { $we_vals[9] = '$1'; } # Windrichtung N
              elsif (($temp > 202.5) && ($temp <= 247.5)) { $we_vals[9] = '$2'; } # Windrichtung NO
              elsif (($temp > 247.5) && ($temp <= 292.5)) { $we_vals[9] = '$3'; } # Windrichtung O
              elsif (($temp > 292.5) && ($temp <= 337.5)) { $we_vals[9] = '$4'; } # Windrichtung SO
              else { $we_vals[9] = '$5'; }                                        # Windrichtung S
            }
          }
          undef $srtime;                                      # Sonnenaufgangszeit löschen
          undef $sstime;                                      # Sonnenuntergangszeit löschen
          $temp3 = $jsonbuf->{'sunrise'};                     # Sonnenaufgang holen
          $temp4 = $jsonbuf->{'sunset'};                      # Sonnenuntergang holen
          if (defined $temp3 && defined $temp4) {             # wenn Werte vorhanden
            if (($temp3 =~ /^\d{10}$/) && ($temp4 =~ /^\d{10}$/)) { # wenn gültige Werte vorhanden
              $srtime = $temp3;                               # Sonnenaufgangszeit setzen
              $sstime = $temp4;                               # Sonnenuntergangszeit setzen
            }
          }
          $we_vals[10] = '$S';                                # Kennung $S voreinstellen
          $temp = $jsonbuf->{'icon'};                         # Wetter-Icon ermitteln
          if (defined $temp) {                                # wenn Wert vorhanden
            if ($temp =~ /^\d{2}[d|n]$/) {                    # wenn plausibler Wert
              if ((defined $srtime) && (defined $sstime)) {   # wenn Sonnenaufgangs- und Sonnenuntergang vorhanden
                if ((time > $srtime) && (time < $sstime)) {   # wenn Zeit innerhalb von Sonnaufgang und Sonnenunterg.
                  $temp = substr($temp, 0, 2) . 'd';          # Icon auf Tag setzen
                }
                else { $temp = substr($temp, 0, 2) . 'n'; }   # sonst Nachtzeit, Icon auf Nacht setzen
              }
              for ($i = 0; $i < @icons; $i ++) {              # Iconliste durchsuchen
                if ($temp eq $icons[$i]) {                    # wenn Icon gefunden
                  $we_vals[10] = '$' . chr($i + ord('A'));    # Icon-Kennung zusammensetzen ($ und Großbuchstabe)
                }
              }
            }
          }
          undef $we_vals[11];                                 # Wetterlagetext löschen
          $temp2 = $jsonbuf->{'id'};                          # Wetterlagenummer ermitteln
          if (defined $temp2) {                               # wenn Wert vorhanden
            for ($i = 0; $i < @wecndn; $i ++) {               # Wetterlagenummer-Liste durchsuchen
              if ($temp2 eq $wecndn[$i]) {                    # wenn Wetterlagenummer gefunden
                $we_vals[11] = $wecndt[$i];                   # entsprechenden Wetterlagetext setzen
              }
            }
          }
          undef $we_vals[12];                                 # Sonnenaufgang/Sonnenuntergang löschen
          if ((defined $srtime) && (defined $sstime)) {       # wenn Sonnenaufgangs- und Sonnenuntergangszeit vorhanden
            $we_vals[12] = (localtime($srtime))[2];           # Stunde des Sonnenaufgangs ermitteln
            if ($we_vals[12] < 10) { $we_vals[12] = '0' . $we_vals[12]; } # wenn Stunde < 10 -> 0 voranstellen
            $temp3 = (localtime($srtime))[1];                 # Minute des Sonnenaufgangs ermitteln
            if ($temp3 < 10) { $temp3 = '0' . $temp3; }       # wenn Minute < 10 -> 0 voranstellen
            $we_vals[12] .= ':' . $temp3;                     # Doppelpunkt und Minute anfügen
            $temp3 = (localtime($sstime))[2];                 # Stunde des Sonnenuntergangs ermitteln
            if ($temp3 < 10) { $temp3 = '0' . $temp3; }       # wenn Stunde < 10 -> 0 voranstellen
            $we_vals[12] .= '$T' . $temp3;                    # Sonnenauf/untergangs-Symbol und Stunde anfügen
            $temp3 = (localtime($sstime))[1];                 # Minute des Sonnenuntergangs ermitteln
            if ($temp3 < 10) { $temp3 = '0' . $temp3; }       # wenn Minute < 10 -> 0 voranstellen
            $we_vals[12] .= ':' . $temp3;                     # Doppelpunkt und Minute anfügen
          }
        }
        if ($test) {                                          # wenn Test aktiv
          for ($i = 0; $i < 13; $i ++) {                      # 13 Sensorwerte der Wetterdaten ausgeben
            print chr(ord('M') + $i), ' ';                    # Sensorkennung ausgeben
            if (defined $we_vals[$i]) {                       # wenn Wert definiert
              print $we_vals[$i];                             # Wert ausgeben
              if ($i == 10 && defined $temp) { print " ($temp)"; }   # wenn Icon -> zusätzlich Nummer ausgeben
              if ($i == 11 && defined $temp2) { print " ($temp2)"; } # wenn Wetterlage -> zusätzlich Nummer ausgeben
              if ($i == 12) { print ' (', substr($we_vals[12], 0, 5), '/', substr($we_vals[12], 7, 5), ')'; } # SA/SU
            }
            print "\n";
          }
        }
      }
      else {                                                  # kein MQTT Topic für Wetterdaten -> direkte Abfrage
        if ($we_use) {                                        # wenn Wetterdatenabfrage aktiviert
          if (-e "$sendir/$jsname") {                         # wenn Wetterdatei vorhanden
            $we_time = (stat "$sendir/$jsname")[9];           # Zeitstempel von Wetterdatei übernehmen
            if (open($WEA, "<$sendir/$jsname")) {             # wenn Wetterdatei vorhanden -> öffnen
              $we_res = <$WEA>;                               # Wetterdaten einlesen
              close($WEA);                                    # Wetterdatei schließen
              if (defined $we_res) { chomp($we_res); }        # wenn Daten vorhanden -> Zeilenendezeichen entfernen
              if ($test) { print "Wetterdaten:\n$we_res\n"; }
            }
          }
          if ($mqtt_use) { $mqtt->tick(0.1); }                # zwischenzeitlich empfangene MQTT-Nachrichten abfragen
          if (defined $we_res) {                              # wenn ein Datenpaket empfangen wurde
            if ((substr($we_res, 0, 1) eq '{') && (substr($we_res, -1, 1) eq '}')) { # wenn JSON-Objekt ({} vorhanden)
              $jsonbuf = decode_json($we_res);                # JSON-Daten dekodieren und speichern
              undef $we_vals[0];                              # Temperatur mit Nachkommastelle löschen
              undef $we_vals[1];                              # Temperatur ohne Nachkommastelle löschen
              $temp = $jsonbuf->{'main'}->{'temp'};           # Temperaturwert ermitteln
              if (defined $temp) {                            # wenn Wert vorhanden
                if ($temp =~ /^\s*\-?\d*\.?\d*$/) {           # wenn plausibler Wert
                  $we_vals[0] = sprintf("%.1f", $temp);       # Temperatur in °C mit einer Nachkommastelle speichern
                  if (($we_vals[0] == 0) && ($we_vals[0] =~ /\-+/)) { # wenn Spezialfall -0 eintritt
                    $temp = 0;                                # Wert auf 0 setzen
                    $we_vals[0] = sprintf("%.1f", $temp);     # und wiederum mit einer Nachkommastelle speichern
                  }
                  $we_vals[1] = sprintf("%.0f", $temp);       # Temperatur in °C ohne Nachkommastelle speichern
                  if (($we_vals[1] == 0) && ($we_vals[1] =~ /\-+/)) { # wenn Spezialfall -0 eintritt
                    $temp = 0;                                # Wert auf 0 setzen
                    $we_vals[1] = sprintf("%.0f", $temp);     # und wiederum ohne Nachkommastelle speichern
                  }
                }
              }
              undef $we_vals[2];                              # Luftfeuchtigkeit löschen
              $temp = $jsonbuf->{'main'}->{'humidity'};       # Luftfeuchtigkeitswert ermitteln
              if (defined $temp) {                            # wenn Wert vorhanden
                if ($temp =~ /^\s*\-?\d*\.?\d*$/) {           # wenn plausibler Wert
                  $we_vals[2] = sprintf("%.0f", $temp);       # Luftfeuchtigkeit in % ohne Nachkommastelle speichern
                }
              }
              undef $we_vals[3];                              # Luftdruck löschen
              $temp = $jsonbuf->{'main'}->{'pressure'};       # Luftdruck ermitteln
              if (defined $temp) {                            # wenn Wert vorhanden
                if ($temp =~ /^\s*\-?\d*\.?\d*$/) {           # wenn plausibler Wert
                  $we_vals[3] = sprintf("%.0f", $temp);       # Luftdruck in hPa ohne Nachkommastelle speichern
                }
              }
              undef $we_vals[4];                              # Windgeschwindigkeit in m/s löschen
              undef $we_vals[5];                              # Windgeschwindigkeit in km/h löschen
              $temp = $jsonbuf->{'wind'}->{'speed'};          # Windgeschwindigkeit ermitteln
              if (defined $temp) {                            # wenn Wert vorhanden
                if ($temp =~ /^\s*\-?\d*\.?\d*$/) {           # wenn plausibler Wert
                  $we_vals[4] = sprintf("%.1f", $temp);       # Windgeschwindigkeit in m/s mit einer Nachkommastelle
                  $we_vals[5] = sprintf("%.0f", $temp * 3.6); # Windgeschwindigkeit in km/h ohne Nachkommastelle
                }
              }
              undef $we_vals[6];                              # Regenmenge in mm/h mit Nachkommastelle löschen
              undef $we_vals[7];                              # Regenmenge in mm/h ohne Nachkommastelle löschen
              $temp = $jsonbuf->{'rain'}->{'1h'};             # Regenmenge ermitteln
              if (defined $temp) {                            # wenn Wert vorhanden
                if ($temp =~ /^\s*\-?\d*\.?\d*$/) {           # wenn plausibler Wert
                  $we_vals[6] = sprintf("%.1f", $temp);       # Regenmenge in mm/h mit einer Nachkommastelle
                  $we_vals[7] = sprintf("%.0f", $temp);       # Regenmenge in mm/h ohne Nachkommastelle
                }
              }
              undef $we_vals[8];                              # Wolkendichte löschen
              $temp = $jsonbuf->{'clouds'}->{'all'};          # Wolkendichte ermitteln
              if (defined $temp) {                            # wenn Wert vorhanden
                if ($temp =~ /^\s*\-?\d*\.?\d*$/) {           # wenn plausibler Wert
                  $we_vals[8] = sprintf("%.0f", $temp);       # Wolkendichte in % ohne Nachkommastelle speichern
                }
              }
              $we_vals[9] = '$0';                             # keine Windrichtung voreinstellen
              $temp = $jsonbuf->{'wind'}->{'deg'};            # Windrichtung ermitteln
              if (defined $temp) {                            # wenn Wert vorhanden
                if ($temp =~ /^\s*\-?\d*\.?\d*$/) {           # wenn plausibler Wert
                  if (($temp > 22.5) && ($temp <= 67.5)) { $we_vals[9] = '$6'; }      # Windrichtung SW
                  elsif (($temp > 67.5) && ($temp <= 112.5)) { $we_vals[9] = '$7'; }  # Windrichtung W
                  elsif (($temp > 112.5) && ($temp <= 157.5)) { $we_vals[9] = '$8'; } # Windrichtung NW
                  elsif (($temp > 157.5) && ($temp <= 202.5)) { $we_vals[9] = '$1'; } # Windrichtung N
                  elsif (($temp > 202.5) && ($temp <= 247.5)) { $we_vals[9] = '$2'; } # Windrichtung NO
                  elsif (($temp > 247.5) && ($temp <= 292.5)) { $we_vals[9] = '$3'; } # Windrichtung O
                  elsif (($temp > 292.5) && ($temp <= 337.5)) { $we_vals[9] = '$4'; } # Windrichtung SO
                  else { $we_vals[9] = '$5'; }                                        # Windrichtung S
                }
              }
              undef $srtime;                                  # Sonnenaufgangszeit löschen
              undef $sstime;                                  # Sonnenuntergangszeit löschen
              $temp3 = $jsonbuf->{'sys'}->{'sunrise'};        # Sonnenaufgang holen
              $temp4 = $jsonbuf->{'sys'}->{'sunset'};         # Sonnenuntergang holen
              if (defined $temp3 && defined $temp4) {         # wenn Werte vorhanden
                if (($temp3 =~ /^\d{10}$/) && ($temp4 =~ /^\d{10}$/)) { # wenn gültige Werte vorhanden
                  $srtime = $temp3;                           # Sonnenaufgangszeit setzen
                  $sstime = $temp4;                           # Sonnenuntergangszeit setzen
                }
              }
              $we_vals[10] = '$S';                            # Kennung $S voreinstellen
              $temp = $jsonbuf->{'weather'}->[0]->{'icon'};   # Wetter-Icon ermitteln
              if (defined $temp) {                            # wenn Wert vorhanden
                if ($temp =~ /^\d{2}[d|n]$/) {                # wenn plausibler Wert
                  if ((defined $srtime) && (defined $sstime)) { # wenn Sonnenaufgangs- und Sonnenuntergang vorhanden
                    if ((time > $srtime) && (time < $sstime)) { # wenn Zeit innerhalb von Sonnaufgang und Sonnenunterg.
                      $temp = substr($temp, 0, 2) . 'd';        # Icon auf Tag setzen
                    }
                    else { $temp = substr($temp, 0, 2) . 'n'; } # sonst Nachtzeit, Icon auf Nacht setzen
                  }
                  for ($i = 0; $i < @icons; $i ++) {          # Iconliste durchsuchen
                    if ($temp eq $icons[$i]) {                # wenn Icon gefunden
                      $we_vals[10] = '$' . chr($i + ord('A')); # Icon-Kennung zusammensetzen ($ und Großbuchstabe)
                    }
                  }
                }
              }
              undef $we_vals[11];                             # Wetterlagetext löschen
              $temp2 = $jsonbuf->{'weather'}->[0]->{'id'};    # Wetterlagenummer ermitteln
              if (defined $temp2) {                           # wenn Wert vorhanden
                for ($i = 0; $i < @wecndn; $i ++) {           # Wetterlagenummer-Liste durchsuchen
                  if ($temp2 eq $wecndn[$i]) {                # wenn Wetterlagenummer gefunden
                    $we_vals[11] = $wecndt[$i];               # entsprechenden Wetterlagetext setzen
                  }
                }
              }
              undef $we_vals[12];                             # Sonnenaufgang/Sonnenuntergang löschen
              if ((defined $srtime) && (defined $sstime)) {   # wenn Sonnenaufgangs- und Sonnenuntergangszeit vorhanden
                $we_vals[12] = (localtime($srtime))[2];       # Stunde des Sonnenaufgangs ermitteln
                if ($we_vals[12] < 10) { $we_vals[12] = '0' . $we_vals[12]; } # wenn Stunde < 10 -> 0 voranstellen
                $temp3 = (localtime($srtime))[1];             # Minute des Sonnenaufgangs ermitteln
                if ($temp3 < 10) { $temp3 = '0' . $temp3; }   # wenn Minute < 10 -> 0 voranstellen
                $we_vals[12] .= ':' . $temp3;                 # Doppelpunkt und Minute anfügen
                $temp3 = (localtime($sstime))[2];             # Stunde des Sonnenuntergangs ermitteln
                if ($temp3 < 10) { $temp3 = '0' . $temp3; }   # wenn Stunde < 10 -> 0 voranstellen
                $we_vals[12] .= '$T' . $temp3;                # Sonnenauf/untergangs-Symbol und Stunde anfügen
                $temp3 = (localtime($sstime))[1];             # Minute des Sonnenuntergangs ermitteln
                if ($temp3 < 10) { $temp3 = '0' . $temp3; }   # wenn Minute < 10 -> 0 voranstellen
                $we_vals[12] .= ':' . $temp3;                 # Doppelpunkt und Minute anfügen
              }
            }
          }
          if ($test) {                                        # wenn Test aktiv
            for ($i = 0; $i < 13; $i ++) {                    # 13 Sensorwerte der Wetterdaten ausgeben
              print chr(ord('M') + $i), ' ';                  # Sensorkennung ausgeben
              if (defined $we_vals[$i]) {                     # wenn Wert definiert
                print $we_vals[$i];                           # Wert ausgeben
                if ($i == 10 && defined $temp) { print " ($temp)"; }   # wenn Icon -> zusätzlich Nummer ausgeben
                if ($i == 11 && defined $temp2) { print " ($temp2)"; } # wenn Wetterlage -> zusätzlich Nummer ausgeben
                if ($i == 12) { print ' (', substr($we_vals[12], 0, 5), '/', substr($we_vals[12], 7, 5), ')'; } # SA/SU
              }
              print "\n";
            }
          }
        }
      }
                                                              # Kraftstoffpreise bearbeiten
      if ($sdtop2 gt '') {                                    # wenn MQTT Topic für Kraftstoffdaten vorhanden
        if ($mqttfval gt '') {                                # wenn eine Datenpaket empfangen wurde
          $jsonbuf = decode_json($mqttfval);                  # JSON-Daten dekodieren und speichern
          for ($i = 0; $i < 4; $i ++) {                       # 4 Array-Elemente lesen
            undef $fu_vals[$i];                               # Kraftstoffpreis löschen
            $temp = $jsonbuf->[$i]->{'price'};                # Preis ermitteln
            if (defined $temp) {                              # wenn Preis vorhanden
              if ($temp =~ /^\d+\.\d+$/) {                    # wenn plausibler Wert
                $temp2 = $jsonbuf->[$i]->{'isOpen'};          # Öffnungsstatus ermitteln
                if (defined $temp2) {                         # wenn Öffnungsstatus vorhanden
                  if ($temp2) {                               # wenn geöffnet
                    $fu_vals[$i] = sprintf("%.2f", $temp);    # Preis mit 2 Nachkommastellen speichern
                  }
                  else { $fu_vals[$i] = "-.--"; }             # wenn geschlossen -> leeren Preis speichern
                }
              }
            }
          }
        }
        if ($test) {                                          # wenn Test aktiv
          for ($i = 0; $i < 4; $i ++) {                       # 4 gespeicherte Preise ausgeben
            print chr(ord('a') + $i), ' ';                    # Sensorkennung ausgeben
            if (defined $fu_vals[$i]) { print $fu_vals[$i]; } # wenn Wert definiert -> ausgeben
            print "\n";
          }
        }
      }
    }
                                                              # Anrufe bearbeiten (in jedem Zyklus)
    if ($call_use > 0) {                                      # wenn Anrufanzeige aktiviert
      if (($call_evnt gt '') && ($call_name gt '') &&
          ($call_extn gt '') && ($call_intn gt '')) {         # wenn alle Anruf-Parameter empfangen
        if ($call_evnt eq 'ring') {                           # wenn Anruf-Ereignis
          if (($call_numb eq '') || ($call_numb eq $call_intn)) { # wenn keine Nummer definiert oder eigene Nummer ok
            if ($call_name eq 'unknown') {                    # wenn Anrufer unbekannt (nicht im Telefonbuch)
              $temp = $call_extn;                             # -> Externe Nummer verwenden
            }
            else {                                            # wenn Anrufer bekannt
              $temp = $call_name;                             # -> Name verwenden
            }
            if ($temp eq 'unknown') {                         # wenn Anrufer ohne Nummer
              $temp = 'unbekannt';                            # -> 'unbekannt' anzeigen
            }
            $almparam[$call_use - 1] = substr($almparam[$call_use - 1], 0, 2) . $temp; # String zusammensetzen
            send_alarm($call_use, 1);                         # Anruf als Alarm signalisieren
            $mqtt->tick(0.1);                                 # 100ms Pause, derweil empfangene MQTT-Nachrichten abfr.
            send_alarm($call_use, 0);                         # Alarm wieder löschen
          }
        }
        $call_evnt = '';                                      # Ereignis wieder löschen
        $call_name = '';                                      # Name wieder löschen
        $call_extn = '';                                      # Externe Nummer wieder löschen
        $call_intn = '';                                      # Interne Nummer wieder löschen
      }
    }

    $lasttime = time;                                         # neuen Zeitstempel für Datenausgabe setzen
    $count ++;                                                # nächstes Datenpaket senden
    if (($count > $senmax + 4) || ($count > 31)) {            # wenn alle Datenpakete gesendet oder Bereich überschr.
      $count = 1;                                             # Zyklus-Zähler wieder auf Anfangswert setzen
      $sencnt = 1;                                            # Sensornummerzähler wieder auf Anfangswert setzen
    }

    if (time > $laststat + 60) {                              # wenn Zeitpunkt für neue Statusausgabe erreicht
      $laststat = time;                                       # aktuellen Zeitstempel speichern
      if ($mqtt_use) {                                        # wenn MQTT eingeschaltet
        if ($mqtt_upt gt '') {                                # wenn MQTT-Topic für Systemlaufzeit vorhanden
          $uptime = '';                                       # Systemlaufzeit zunächst löschen
          $tstr = `cat /proc/uptime`;                         # Systemlaufzeit aus System auslesen
          for ($i = 0; $i < 16; $i ++) {                      # maximal 16 Zeichen lesen
            if (substr($tstr, $i, 1) gt ' ') {                # wenn Zeichencode größer als Leerzeichen
              $uptime .= substr($tstr, $i, 1);                # Zeichen an Systemlaufzeit anfügen
            } else { last; }                                  # wenn Leerzeichen -> Schleife abbrechen
          }
          $uptime = sprintf("%.2f", $uptime / 60 / 60 / 24);  # Systemlaufzeit in Stunden ermitteln
          $mqtt->publish($mqtt_upt, $uptime);                 # Systemlaufzeit an MQTT-Server senden
        }
        if ($mqtt_rssi gt '') {                               # wenn MQTT-Topic für RSSI-Wert vorhanden
          $rssival = '';                                      # RSSI-Wert zunächst löschen
          $tstr = `iwconfig wlan0`;                           # WLAN-Status ermitteln
          if ($tstr =~ /level=/g) {                           # wenn Schlüsselwort "level=" gefunden
            for ($i = $-[0]+6; $i < $-[0]+10; $i ++) {        # 4 Zeichen von RSSI-Wert lesen
              if (substr($tstr, $i, 1) gt ' ') {              # wenn Zeichencode größer als Leerzeichen
                $rssival .= substr($tstr, $i, 1);             # Zeichen an RSSI-Wert anfügen
              } else { last; }                                # wenn Leerzeichen -> Schleife abbrechen
            }
          }
          $mqtt->publish($mqtt_rssi, $rssival);               # RSSI-Wert an MQTT-Server senden
        }
        if ($mqtt_load gt '') {                               # wenn MQTT-Topic für Systemlast vorhanden
          $loadval = '';                                      # Systemlast zunächst löschen
          $tstr = `cat /proc/loadavg`;                        # Systemlast aus System auslesen
          for ($i = 0; $i < 6; $i ++) {                       # 6 Zeichen von Systemlast lesen
            if (substr($tstr, $i, 1) gt ' ') {                # wenn Zeichencode größer als Leerzeichen
              $loadval .= substr($tstr, $i, 1);               # Zeichen an Systemlast anfügen
            } else { last; }                                  # wenn Leerzeichen -> Schleife abbrechen
          }
          $mqtt->publish($mqtt_load, $loadval);               # Systemlast an MQTT-Server senden
        }
      }
    }
  }
}

# Unterprogramme zum Verarbeiten der empfangenen MQTT-Publishes der Wetterdaten, der Kraftstoffdaten, sowie der Werte
# der 12 Sensoren, 4 Alarme und 4 Anrufparameter

sub sdtopic1 {                                                # Callback-Aufruf für Wetterdaten
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  if ($message gt '') {                                       # wenn ein Datenpaket empfangen wurde
    if ((substr($message, 0, 1) eq '{') && (substr($message, -1, 1) eq '}')) { # wenn JSON-Objekt {}
      $mqttwval = $message;                                   # Wetterdaten zwischenspeichern
      $we_time = time;                                        # Zeitstempel speichern
      if ($test) { print "Wetterdaten:\n$mqttwval\n"; }
    }
  }
}

sub sdtopic2 {                                                # Callback-Aufruf für Kraftstoffdaten
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  if ($message gt '') {                                       # wenn ein Datenpaket empfangen wurde
    if ((substr($message, 0, 1) eq '[') && (substr($message, -1, 1) eq ']')) { # wenn JSON-Array []
      $mqttfval = $message;                                   # Kraftstoffdaten zwischenspeichern
      $mqttftim = time;                                       # Zeitstempel speichern
      if ($test) { print "Kraftstoffdaten:\n$mqttfval\n"; }
    }
  }
}

sub sdtopic3 {                                                # Callback-Aufruf für Nachrichten
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  if ($message gt '') {                                       # wenn ein Datenpaket empfangen wurde
    $mqttmval = $message;                                     # Nachricht zwischenspeichern
    if ($test) { print "Nachricht: $mqttmval\n"; }
  }
}

sub sensorA {                                                 # Callback-Aufruf für Sensor A
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[0]);            # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[0] = $tempval;                                   # Sensorwert speichern
    $sentimes[0] = time;                                      # Zeitstempel speichern
    if ($test) { print "A $tempval\n"; }
  }
}
sub sensorB {                                                 # Callback-Aufruf für Sensor B
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[1]);            # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[1] = $tempval;                                   # und speichern
    $sentimes[1] = time;                                      # Zeitstempel speichern
    if ($test) { print "B $tempval\n"; }
  }
}
sub sensorC {                                                 # Callback-Aufruf für Sensor C
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[2]);            # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[2] = $tempval;                                   # und speichern
    $sentimes[2] = time;                                      # Zeitstempel speichern
    if ($test) { print "C $tempval\n"; }
  }
}
sub sensorD {                                                 # Callback-Aufruf für Sensor D
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[3]);            # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[3] = $tempval;                                   # und speichern
    $sentimes[3] = time;                                      # Zeitstempel speichern
    if ($test) { print "D $tempval\n"; }
  }
}
sub sensorE {                                                 # Callback-Aufruf für Sensor E
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[4]);            # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[4] = $tempval;                                   # und speichern
    $sentimes[4] = time;                                      # Zeitstempel speichern
    if ($test) { print "E $tempval\n"; }
  }
}
sub sensorF {                                                 # Callback-Aufruf für Sensor F
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[5]);            # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[5] = $tempval;                                   # und speichern
    $sentimes[5] = time;                                      # Zeitstempel speichern
    if ($test) { print "F $tempval\n"; }
  }
}
sub sensorG {                                                 # Callback-Aufruf für Sensor G
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[6]);            # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[6] = $tempval;                                   # und speichern
    $sentimes[6] = time;                                      # Zeitstempel speichern
    if ($test) { print "G $tempval\n"; }
  }
}
sub sensorH {                                                 # Callback-Aufruf für Sensor H
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[7]);            # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[7] = $tempval;                                   # und speichern
    $sentimes[7] = time;                                      # Zeitstempel speichern
    if ($test) { print "H $tempval\n"; }
  }
}
sub sensorI {                                                 # Callback-Aufruf für Sensor I
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[8]);            # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[8] = $tempval;                                   # und speichern
    $sentimes[8] = time;                                      # Zeitstempel speichern
    if ($test) { print "I $tempval\n"; }
  }
}
sub sensorJ {                                                 # Callback-Aufruf für Sensor J
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[9]);            # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[9] = $tempval;                                   # und speichern
    $sentimes[9] = time;                                      # Zeitstempel speichern
    if ($test) { print "J $tempval\n"; }
  }
}
sub sensorK {                                                 # Callback-Aufruf für Sensor K
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[10]);           # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[10] = $tempval;                                  # und speichern
    $sentimes[10] = time;                                     # Zeitstempel speichern
    if ($test) { print "K $tempval\n"; }
  }
}
sub sensorL {                                                 # Callback-Aufruf für Sensor L
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  my $tempval = check_val($message, $senflist[11]);           # Sensorwert prüfen und konvertieren
  if ($tempval gt '') {                                       # wenn Sensorwert in Ordnung
    $senvals[11] = $tempval;                                  # und speichern
    $sentimes[11] = time;                                     # Zeitstempel speichern
    if ($test) { print "L $tempval\n"; }
  }
}
sub alarm1 {                                                  # Callback-Aufruf für Alarm 1
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  send_alarm(1, $message);                                    # Alarmmeldung senden
}
sub alarm2 {                                                  # Callback-Aufruf für Alarm 2
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  send_alarm(2, $message);                                    # Alarmmeldung senden
}
sub alarm3 {                                                  # Callback-Aufruf für Alarm 3
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  send_alarm(3, $message);                                    # Alarmmeldung senden
}
sub alarm4 {                                                  # Callback-Aufruf für Alarm 4
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  send_alarm(4, $message);                                    # Alarmmeldung seden
}
sub call1 {                                                   # Callback-Aufruf für Anruf (Ereignis)
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  $call_evnt = $message;                                      # und speichern
}
sub call2 {                                                   # Callback-Aufruf für Anruf (Name)
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  $temp = decode('UTF-8', $message, Encode::FB_DEFAULT);      # empfangenen String in Perl-Format wandeln
  $temp = substr($temp, 0, 12);                               # String auf 12 Zeichen kürzen
  $call_name = encode('UTF-8', $temp, Encode::FB_DEFAULT);    # wieder in UTF-8 wandeln und speichern
}
sub call3 {                                                   # Callback-Aufruf für Anruf (Externe Nummer)
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  $call_extn = substr($message, 0, 10);                       # auf 10 Zeichen kürzen und speichern
  if (length($message) > 10) { $call_extn .= '.'; }           # falls Nummer länger als 10 Zeichen -> Punkt anhängen
}
sub call4 {                                                   # Callback-Aufruf für Anruf (Interne Nummer)
  my ($topic, $message) = @_;                                 # empfangene Daten lesen
  $call_intn = $message;                                      # und speichern
}

# Unterprogramm zum Prüfen und Konvertieren von Sensorwerten, die über MQTT empfangen wurden
# Parameter 1: Sensorwert
# Parameter 2: Format (0-2 Anzahl der Nachkommastellen, >2 keine Prüfung)
# Rückgabe:    geprüfter und konvertierter Wert (wenn Fehler: '')

sub check_val {
  my $ret = '';                                               # leeren Rückgabewert voreinstellen
  if ($_[1] == 0) {                                           # wenn Format = 0 (keine Nachkommastelle)
    if ($_[0] =~ /^\s*\-?\d*\.?\d*$/) {                       # wenn Sensorwert plausibel ist
      if (length($_[0]) > 0) {                                # wenn Länge mindestens 1 Zeichen
        $ret = sprintf("%.0f", $_[0]);                        # Wert ohne Nachkommastelle zurückgeben
        if (($ret == 0) && ($ret =~ /\-+/)) {                 # wenn Spezialfall -0 eintritt
          $ret = sprintf("%.0f", 0);                          # auf 0 setzen und ohne Nachkommastelle zurückgeben
        }
      }
    }
  }
  elsif ($_[1] == 1) {                                        # wenn Format = 1 (eine Nachkommastelle)
    if ($_[0] =~ /^\s*\-?\d*\.?\d*$/) {                       # wenn Sensorwert plausibel ist
      if (length($_[0]) > 0) {                                # wenn Länge mindestens 1 Zeichen
        $ret = sprintf("%.1f", $_[0]);                        # Wert mit einer Nachkommastelle zurückgeben
        if (($ret == 0) && ($ret =~ /\-+/)) {                 # wenn Spezialfall -0 eintritt
          $ret = sprintf("%.1f", 0);                          # auf 0 setzen und mit einer Nachkommastelle zurückgeben
        }
      }
    }
  }
  elsif ($_[1] == 2) {                                        # wenn Format = 2 (zwei Nachkommastellen)
    if ($_[0] =~ /^\s*\-?\d*\.?\d*$/) {                       # wenn Sensorwert plausibel ist
      if (length($_[0]) > 0) {                                # wenn Länge mindestens 1 Zeichen
        $ret = sprintf("%.2f", $_[0]);                        # Wert mit zwei Nachkommastelle zurückgeben
        if (($ret == 0) && ($ret =~ /\-+/)) {                 # wenn Spezialfall -0 eintritt
          $ret = sprintf("%.2f", 0);                          # auf 0 setzen und mit zwei Nachkommastellen zurückgeben
        }
      }
    }
  }
  elsif ($_[1] > 2) {                                         # wenn Format > 2 (keine Konvertierung)
    if (length($_[0]) > 0) { $ret = $_[0]; }                  # wenn Länge mindestens 1 Zeichen -> direkt zurückgeben
  }
  return $ret;                                                # Ergebnis zurückgeben
}

# Unterprogramm zum Senden einer Alarmmeldung (Alarme 1-4 des Sensormoduls und spezielle Alarme 5-8)
# Parameter 1: Alarmnummer (1-8)
# Parameter 2: Alarmstatus (0-1)

sub send_alarm {                                              # Alarmstatus senden
  if ($_[1] =~ /^[0-1]$/) {                                   # wenn Alarm-Status Ziffer 0 oder 1
    my $param = $almparam[$_[0] - 1];                         # Alarmparameter holen
    if ($param gt '') {                                       # wenn Alarmparameter vorhanden
      if ($_[1] == 1) {                                       # wenn Alarm aktiv
        print $SER "A$_[0]:$_[1]$param\n";                    # vollständigen Alarmtext ausgeben
        if ($test) { print "A$_[0]:$_[1]$param\n"; }
      }
      else {                                                  # wenn Alarm inaktiv
        print $SER "A$_[0]:$_[1]\n";                          # nur Alarmstatus ausgeben
        if ($test) { print "A$_[0]:$_[1]\n"; }
      }
    }
  }
}
